import {
  resample,
  stepState,
  stateConverged,
  PenroseState,
  RenderStatic,
} from "@penrose/core";
import * as React from "react";
import styled from "styled-components";
import Logo from "ui/icons/Logo";
import Resample from "ui/icons/Resample";

const EmbedContainer = styled.div`
  position: relative;
  border-radius: 10px;
  border: 0.5px solid rgba(0, 0, 0, 0.2);
  box-shadow: 0 5px 8px 0 rgba(0, 0, 0, 0.2);
  overflow: hidden;
`;

const EmbedFooter = styled.div`
  display: flex;
  overflow: hidden;
  justify-content: space-between;
  padding: 0 1em;
  align-items: center;
  background-color: #40b4f7;
  border-radius: 0 0 9px 9px;
`;

const Text = styled.p`
  font-family: "Open Sans", sans-serif;
  color: white;
  margin: 0.7em;
  font-weight: lighter;
  font-size: smaller;
`;

interface IEmbedState {
  data: PenroseState;
}

class Embed extends React.Component<any, IEmbedState> {
  // NOTE: assume an initial state passed in upon mounting
  constructor(props: IEmbedState) {
    super(props);
    this.state = { data: props.data };
    // step until convergence
    this.step();
  }

  public onCanvasState = async (canvasState: PenroseState) => {
    // HACK: this will enable the "animation" that we normally expect
    await new Promise((r) => setTimeout(r, 1));
    this.setState({
      data: canvasState,
    });
    if (!stateConverged(canvasState)) {
      this.step();
    }
  };

  public step = (): void => {
    const stepped = stepState(this.state.data!);
    void this.onCanvasState(stepped);
  };

  public resample = (): void => {
    const NUM_SAMPLES = 1;
    const oldState = this.state.data;
    if (oldState) {
      const resampled = resample(oldState, NUM_SAMPLES);
      void this.onCanvasState(resampled);
    }
  };

  public updateData = async (data: PenroseState) => {
    this.setState({ data: { ...data } });
    const stepped = stepState(data);
    void this.onCanvasState(stepped);
  };

  public render() {
    const { data } = this.state;
    return (
      <EmbedContainer>
        <EmbedFooter>
          {data && (
            <div
              style={{ width: "100%", height: "100%" }}
              dangerouslySetInnerHTML={{
                __html: RenderStatic(data.shapes, data.labelCache).outerHTML,
              }}
            />
          )}
          <Logo width={24} color={"white"} />
          {/* <div style={{ display: "flex", alignItems: "center" }}>
            <Text> Generated by Penrose with</Text>
            <Illuminati width={24} color={"white"} />
          </div> */}
          <Text>
            Generated by Penrose with{" "}
            <span role="img" aria-label="heart">
              💜
            </span>
          </Text>
          <div onClick={this.resample}>
            <Resample size={24} color={"white"} />
          </div>
        </EmbedFooter>
      </EmbedContainer>
    );
  }
}

export default Embed;
